from abc import ABCMeta, abstractmethod
from typing import List

from .._sources.base_classes import FinancialSymbolsSource
from .._sources.inflation_source import InflationSource
from .._sources.micex_stocks_source import MicexStocksSource
from .._sources.moex_indexes_source import MoexIndexesSource
from .._sources.mutru_funds_source import MutualFundsRuSource
from .._sources.okama_source import OkamaSource
from .._sources.us_data_source import UsDataSource
from .._sources.single_financial_symbol_source import CbrCurrenciesSource, CbrTopRatesSource
from .._sources.yahoo_indexes_source import YahooIndexesSource


class SymbolSources(metaclass=ABCMeta):
    @property
    @abstractmethod
    def sources(self) -> List[FinancialSymbolsSource]:
        raise NotImplementedError()


class AllSymbolSources(SymbolSources):

    def __init__(self,
                 cbr_currencies_source: CbrCurrenciesSource,
                 cbr_top_rates_source: CbrTopRatesSource,
                 inflation_source: InflationSource,
                 micex_stocks_source: MicexStocksSource,
                 moex_indexes_source: MoexIndexesSource,
                 mutual_funds_ru_source: MutualFundsRuSource,
                 us_data_source: UsDataSource,
                 okama_source: OkamaSource,
                 yahoo_indexes_source: YahooIndexesSource):
        self.cbr_currencies_source = cbr_currencies_source
        self.cbr_top_rates_source = cbr_top_rates_source
        self.inflation_source = inflation_source
        self.micex_stocks_source = micex_stocks_source
        self.moex_indexes_source = moex_indexes_source
        self.mutual_funds_ru_source = mutual_funds_ru_source
        self.us_data_source = us_data_source
        self.okama_source = okama_source
        self.yahoo_indexes_source = yahoo_indexes_source

    @property
    def sources(self):
        return [
            self.cbr_currencies_source,
            self.cbr_top_rates_source,
            self.inflation_source,
            self.micex_stocks_source,
            self.moex_indexes_source,
            self.mutual_funds_ru_source,
            self.us_data_source,
            self.okama_source,
            self.yahoo_indexes_source,
        ]



from functools import lru_cache
from typing import Optional, Callable

import pandas as pd

from .base_classes import FinancialSymbolsSource
from .._settings import data_url
from ..common.enums import Currency, SecurityType, Period
from ..common.financial_symbol import FinancialSymbol
from ..common.financial_symbol_id import FinancialSymbolId
from ..common.financial_symbol_info import FinancialSymbolInfo


class UsDataSource(FinancialSymbolsSource):
    def __init__(self):
        super().__init__(namespace='ny')

        self.url_base = data_url + 'v2/us'
        self.index = pd.read_csv(self.url_base, sep=',', index_col='Code')

    @lru_cache(maxsize=512)
    def __extract_values(self, name: str) -> Callable[[pd.Period, pd.Period], pd.DataFrame]:
        def func(start_period: pd.Period, end_period: pd.Period) -> pd.DataFrame:
            nonlocal name
            df = pd.read_csv(self.url_base + '/' + name, sep='\t', parse_dates=['period'])
            df.rename(columns={'period': 'date'}, inplace=True)
            df['period'] = df['date'].dt.to_period(freq='M')
            df_new = df[(start_period <= df['period']) & (df['period'] <= end_period)].copy()
            return df_new

        return func

    def fetch_financial_symbol(self, name: str) -> Optional[FinancialSymbol]:
        if name in self.index.index:
            row = self.index.loc[name]
            symbol = FinancialSymbol(identifier=FinancialSymbolId(namespace=self.namespace, name=name),
                                     values=self.__extract_values(name),
                                     exchange=row['Exchange'],
                                     short_name=row['Name'],
                                     currency=Currency.__dict__.get(row['Currency']),  # type: ignore
                                     security_type=SecurityType.STOCK_ETF,
                                     period=Period.MONTH,
                                     adjusted_close=True)
            return symbol
        return None

    def get_all_infos(self):
        infos = []
        for idx, row in self.index.iterrows():
            fsi = FinancialSymbolId(self.namespace, str(idx))
            fs_info = FinancialSymbolInfo(fin_sym_id=fsi, short_name=idx)
            infos.append(fs_info)
        return infos



import pinject

from ._instance import Yapo
from ._sources.all_sources import AllSymbolSources


class BindingSpec(pinject.BindingSpec):
    def configure(self, bind):
        bind('symbol_sources', to_class=AllSymbolSources)


obj_graph = pinject.new_object_graph(binding_specs=[BindingSpec()])
yapo_instance = obj_graph.provide(Yapo)

information = yapo_instance.information
portfolio = yapo_instance.portfolio
portfolio_asset = yapo_instance.portfolio_asset
available_names = yapo_instance.available_names
search = yapo_instance.search
inflation = yapo_instance.inflation
